#include    <Windows.h>
#include    <stdio.h>
#include    "MTypes.h"
#include    "STDmacro.h"
#include    "CAXXCom.h"
/*######################################################################*/
CAXXCom::CAXXCom()
{
    m_pCOM = 0;
    m_nDevice = 0;
}
/*----------------------------------------------------------------------*/
CAXXCom::~CAXXCom()
{
    ExitCOM();
}
/*----------------------------------------------------------------------*/
void    CAXXCom::ExitCOM()
{
    if(m_pCOM)
    {
        m_pCOM->Close();
        m_pCOM->Release();
        m_pCOM = 0;
    }
}
/*----------------------------------------------------------------------*/
int     CAXXCom::Look()
{
    if(m_pCOM == 0)
        return -1;
    if(m_pCOM->Look(LOOK_CONNECT) <= 0)
        return -2;
    return m_nDevice;
}
/*----------------------------------------------------------------------*/
void    CAXXCom::Clear()
{
    if(m_pCOM)
        m_pCOM->Clear();
}
/*======================================================================*/
int     CAXXCom::SetupCOM(int nCOM)
{
    ExitCOM();

    m_pCOM = ICommLightCreate();
    if(m_pCOM == 0) 
        return -1;
    if(m_pCOM->Open(nCOM+1, 4800, 8, 0, 0, 0) <= 0) 
        return 0;

    m_pCOM->Clear();
    m_pCOM->EscapeFunction(SETDTR);
    m_pCOM->EscapeFunction(CLRRTS);
    return 1;
}
/*----------------------------------------------------------------------*/
int     CAXXCom::SendCommand(int devN, char* str, unsigned tout, int val, double dval)
{
    if(m_pCOM == 0 || m_pCOM->Look(LOOK_CONNECT) <= 0 || str == 0)
        return 0;

    while(m_pCOM->Get() != 0);

    if(devN >= 0 && devN < 256)
        m_nDevice = devN;

    char buffw[30], buffr[30];
    if(val >= 0)
        sprintf(buffw, "$%02d%s%d;", m_nDevice, str, val);
    else
    if(dval >= 0.0)
        sprintf(buffw, "$%02d%s%.4f;", m_nDevice, str, dval);
    else
        sprintf(buffw, "$%02d%s;",   m_nDevice, str);

    strupr(buffw);
    int n = strlen(buffw);
    m_pCOM->Write(buffw, n, 1000);
    buffw[--n] = 0;
    DBGF("CAXXCom", "Command = %s", buffw);

    int i = 0;
    if(tout)
    {
        int f = -1;
        unsigned time0 = GetTickCount();

        int c;
        while(1)
        {
            if(GetTickCount() - time0 > tout)
                return f;

            c = m_pCOM->Get();
            if(c == 0)
            {
                Sleep(5);
                continue;
            }
            c &= 0xFF;

            if(c != ';' && c >= ' ')
                break;
        }
        while(1)
        {
            if(c == ';' || c < ' ')
                break;
            buffr[i++] = c;
            if(c == '$')
                f = 1;

            while(1)
            {
                if(GetTickCount() - time0 > tout)
                    return f;

                c = m_pCOM->Get();
                if(c != 0)
                    break;
                Sleep(5);
            }
            c &= 0xFF;
        }
        if(f > 0)
        {
            for(int j=0; j<i && j<n; ++j)
                if(buffw[j] != buffr[j])
                    break;
            if(j == i && j == n)
                ++f;
        }
        buffr[i] = 0;

        DBGF("CAXXCom", "Command = %s, echo = %s, ret = %d", buffw, buffr, f);
        return f;
    }

    DBGF("CAXXCom", "NO echo!");
    return 1;
}
/*----------------------------------------------------------------------*/
int     CAXXCom::ReadInfo(char* buff, int size, unsigned tout)
{
    if(m_pCOM == 0 || m_pCOM->Look(LOOK_CONNECT) <= 0 || buff == 0 || size <= 0)
        return 0;

    buff[0] = 0;

    if(tout == 0)
        return -1;

    int f = -1;
    unsigned time0 = GetTickCount();
    while(1)
    {
        if(GetTickCount() - time0 > tout)
            return f;
      
        int c = m_pCOM->Get();
        if(c == 0) 
        {
            Sleep(5);
            continue;
        }
        c &= 0xFF;

        if(f < 0)
        {
            if(c == '#')
                f = 0;
        }
        else
        {
            if(c == ';' || c == '\n' || c == '\r')
                break;
            if(f >= 0)
            {
                if(f >= size)
                    return -1;
                buff[f++] = toupper(c); 
            }
        }
    }
    buff[f] = 0;

    DBGF("CAXXCom", "Buffer = %s", buff);
    return 1;
}
/*######################################################################*/
