#include <windows.h>
#include <tchar.h>

#include <stdio.h>
#include <stdarg.h>

#include "MTypes.h"
#include "STDMacro.h"
#include "Chunk.h"
#include "CommLight.h"
#include "MInterlocked.h"
/*######################################################################*/
// Written by Andry Zharov
// 20.08.1998
/*######################################################################*/
struct  MChar
{
    MChar*          m_pNext;
    unsigned        m_uData;
};
/*----------------------------------------------------------------------*/
class   MCycleBuff
{
private:
    volatile MChar*         m_pSet;
    volatile MChar*         m_pGet;
    CHUNK_HEAP<MChar, 1024> m_chData;

public:
    MCycleBuff() 
    {
        Erase();
    }

    void        Set(unsigned char c)
    {
        MChar* mc = (MChar*)m_pSet;
        mc->m_uData = unsigned(c);
        if(mc->m_pNext == m_pGet) 
        {
            MChar* smc = mc->m_pNext;
            mc->m_pNext = m_chData.Get(); 
            mc->m_pNext->m_pNext = smc;
        }
        m_pSet = m_pSet->m_pNext;
    }
    
    unsigned    Get()
    {
        if(m_pGet == m_pSet) 
            return 0;
        MChar* mc = (MChar*)m_pGet;
        unsigned c = mc->m_uData | 0x80000000;
        m_pGet = mc->m_pNext;
        return c;
    }
    
    unsigned    Test(int n)
    {
        MChar* mc = (MChar*)m_pGet;
        for(int i = 0; i < n && mc != m_pSet; ++i) 
            mc = mc->m_pNext;
        return (mc == m_pSet) ? 0 : mc->m_uData | 0x80000000;
    }
    
    void        Erase();
};
/*======================================================================*/
void    MCycleBuff::Erase()
{
    m_chData.Erase();
    MChar* mc = m_chData.Get();
    MChar* fmc = mc;
    for(int i = 0; i < 512; ++i, mc = mc->m_pNext) 
        mc->m_pNext = m_chData.Get();
    mc->m_pNext = fmc;
    m_pSet = m_pGet = fmc;
}
/*######################################################################*/
#define ASCII_XON       0x11
#define ASCII_XOFF      0x13
#define CHAR_DELAY      10
/*----------------------------------------------------------------------*/
class   DCommLight : public ICommLight
{
private:
    unsigned            m_uRefCount;

private:
    HANDLE              m_hPort;

    HANDLE              m_hReadThread;
    DWORD               m_dwReadThreadID;
    OVERLAPPED          m_osRead;    
    OVERLAPPED          m_osWrite;    
    CRITICAL_SECTION    m_cs;

    int                 m_nPort;
    int                 m_nByteDelay;

    int                 m_nBaudRate;
    int                 m_nBits;
    int                 m_nStop;
    int                 m_nParity;
    int                 m_nFlowControl;
    unsigned char       m_cXoff, m_cXon;
    DWORD               m_nLastError;

    volatile int        m_nConnected;
    
    MCycleBuff          m_cbRead;

    char                m_pName[10];
    unsigned char       m_cReadBuff[1024];

protected:
    int         Setup(int baud, int bit, int stop, int parity, int flowF);
    friend      DWORD WINAPI CommReadProc(LPSTR);
    unsigned    ARead();
    void        ErrorHandler(int mode);

public:
    DCommLight();
    ~DCommLight();

// Interfaces
    unsigned __stdcall  AddRef();
    unsigned __stdcall  Release();

    int      __stdcall  Open(int pNum, int baud, int bit, int stop, int parity, int flow);
    void     __stdcall  Close();
    
    int      __stdcall  Write(char* buff, int cnt, unsigned tout);
    int      __stdcall  Read( char* buff, int cnt, unsigned tout);
    unsigned __stdcall  Get();
    unsigned __stdcall  Test(int i);

    void     __stdcall  Clear();
    void     __stdcall  EscapeFunction(unsigned fun);
    unsigned __stdcall  GetModemStatus();

    unsigned __stdcall  Look(int code);
};
/*======================================================================*/
void        DCommLight::ErrorHandler(int mode)
{
    DWORD ef; 
    ClearCommError(m_hPort, &ef, 0);
    DBGF("CommLight", "ERROR [%08X], ErrorCode=%08X", this, ef);
    switch(mode)
    {
    case 2:
        {
            if(ef & CE_RXPARITY) 
                m_nLastError = CL_READ_ERROR_RXPARITY;         
            else
            if(ef & CE_FRAME)    
                m_nLastError = CL_READ_ERROR_FRAME;
            else
            if(ef & CE_OVERRUN)  
                m_nLastError = CL_READ_ERROR_OVERRUN;
            PurgeComm(m_hPort, PURGE_RXCLEAR|PURGE_TXCLEAR);
            FlushFileBuffers(m_hPort);
        }
        break;
    case 1: m_nLastError = CL_READ_ERROR;  
            PurgeComm(m_hPort, PURGE_RXCLEAR);
        break;
    case 0: m_nLastError = CL_WRITE_ERROR; 
            PurgeComm(m_hPort, PURGE_TXCLEAR);
        break;
    }
}
/*======================================================================*/
unsigned    DCommLight::ARead()
{
    COMSTAT cs; DWORD ef;
    ClearCommError(m_hPort, &ef, &cs);
    unsigned cnt = (cs.cbInQue > DIM(m_cReadBuff)) ? DIM(m_cReadBuff) : cs.cbInQue;
    if(cnt == 0)
        return 0;
    
    unsigned rcnt = cnt;
    int rs = ReadFile(m_hPort, m_cReadBuff, cnt, (DWORD*)&rcnt, &m_osRead);
    if(!rs)
    {
        if(GetLastError() == ERROR_IO_PENDING)
        {
            while(m_nConnected && !GetOverlappedResult(m_hPort, &m_osRead, (DWORD*)&rcnt, FALSE))
            {
                if(GetLastError() == ERROR_IO_INCOMPLETE) 
                { 
                    Sleep(0);
                    continue; 
                }
                ErrorHandler(1); 
                break;
            }
        }
        else
        {
            ErrorHandler(1);
            return 0;
        }
    }
    if(m_nConnected && (rcnt > 0))
    {
        for(unsigned i = 0; i < rcnt; ++i) 
            m_cbRead.Set(m_cReadBuff[i]);
#ifdef  FULL_DEBUG
        DBGF("CommLight", "Read [%08X], len=%d", this, rcnt);
#endif
        return rcnt;
    }
    return 0;
}
/*----------------------------------------------------------------------*/
DWORD WINAPI CommReadProc(LPSTR lpData)
{
    DCommLight* p = (DCommLight*)lpData;
    DBGF("CommLight", "CommReadProc begin [%08X]", p);
    if(p)
    {
        while(p->m_nConnected)
        {
            DWORD em = 0;
            WaitCommEvent(p->m_hPort, &em, 0);
            if(em & EV_ERR) 
                p->ErrorHandler(2);
            else
            {
                if((em & EV_RXCHAR) != EV_RXCHAR) 
                    continue;
#ifdef  READCOM_STRATEGY1
                while(p->m_nConnected) 
                    if(p->ARead() == 0) 
                        break;
#else
                p->ARead();
#endif
            }
            if(p->m_nConnected) 
                Sleep(p->m_nByteDelay);
        }
    }
    DBGF("CommLight", "CommReadProc end [%08X]", p);
    return 0;
}
/*======================================================================*/
DCommLight::DCommLight()
{
    m_uRefCount = 0;

    m_nConnected = 0;
    m_hPort = INVALID_HANDLE_VALUE;
    m_pName[0] = 0;
    m_nLastError = 0;
    
    m_hReadThread    = 0; 
    m_dwReadThreadID = 0;

    memset(&m_osWrite, 0, sizeof(m_osWrite));
    memset(&m_osRead,  0, sizeof(m_osRead));
    InitializeCriticalSection(&m_cs);

    m_nPort        = 0;
    m_nByteDelay   = CHAR_DELAY;

    m_nBaudRate    = 9600;
    m_nBits        = 8;
    m_nStop        = 0;
    m_nParity      = 0;
    m_nFlowControl = FC_RTSCTS;
    m_cXoff        = ASCII_XOFF;
    m_cXon         = ASCII_XON;

    DBGF("CommLight", "CommLight [%08X]", this);
}
/*----------------------------------------------------------------------*/
DCommLight::~DCommLight()
{
    Close();
    DeleteCriticalSection(&m_cs);
    DBGF("CommLight", "~CommLight [%08X]", this);
}
/*----------------------------------------------------------------------*/
int         DCommLight::Setup(int baud, int bit, int stop, int parity, int flowF)
{
    DBGF("CommLight", "Setup [%08X], Baud=%d, Bits=%d, Stop=%d, Parity=%d, Flow=%X",
         this, baud, bit, stop, parity, flowF);

    COMMPROP prop; prop.wPacketLength = sizeof(prop);
    GetCommProperties(m_hPort, &prop); 
    switch(prop.dwMaxBaud)
    {
    case BAUD_075    : m_nBaudRate =        75; break;
    case BAUD_110    : m_nBaudRate =       110; break;  
    case BAUD_134_5  : m_nBaudRate =       134; break; 
    case BAUD_150    : m_nBaudRate =       150; break; 
    case BAUD_300    : m_nBaudRate =       300; break; 
    case BAUD_600    : m_nBaudRate =       600; break; 
    case BAUD_1200   : m_nBaudRate =      1200; break; 
    case BAUD_1800   : m_nBaudRate =      1800; break; 
    case BAUD_2400   : m_nBaudRate =      2400; break; 
    case BAUD_4800   : m_nBaudRate =      4800; break; 
    case BAUD_7200   : m_nBaudRate =      7200; break; 
    case BAUD_9600   : m_nBaudRate =      9600; break; 
    case BAUD_14400  : m_nBaudRate =     14400; break; 
    case BAUD_19200  : m_nBaudRate =     19200; break; 
    case BAUD_38400  : m_nBaudRate =     38400; break; 
    case BAUD_56K    : m_nBaudRate =     56000; break; 
    case BAUD_57600  : m_nBaudRate =     57600; break; 
    case BAUD_115200 : m_nBaudRate =    115200; break; 
    case BAUD_128K   : m_nBaudRate =    128000; break; 
    default          : 
    case BAUD_USER   : m_nBaudRate =   1048576; break; 
    }
    
    m_cXoff = m_cXon = 0;
    if(flowF & 0x80000000)
    {
        m_cXon  = unsigned char((stop   >> 24) & 0xFF); 
        if(!m_cXon)  
            m_cXon  = ASCII_XON;
        m_cXoff = unsigned char((parity >> 24) & 0xFF);
        if(!m_cXoff)
            m_cXoff = ASCII_XOFF;
    }
    stop   &= 0xFF;
    parity &= 0xFF;
    m_nFlowControl = flowF & (FC_DTRDSR|FC_RTSCTS|FC_XONXOFF);    
    Cut(baud, 75, m_nBaudRate); m_nBaudRate = baud;
    Cut(bit,    4, 8);          m_nBits = bit;
    Cut(stop,   0, 2);          m_nStop = stop;
    Cut(parity, 0, 4);          m_nParity = parity;    

    DCB dcb; dcb.DCBlength = sizeof(dcb);
    GetCommState(m_hPort, &dcb);

    dcb.fBinary = 1;                            // binary mode, no EOF check
    dcb.fParity = 1;                            // enable parity checking 
    dcb.fDsrSensitivity = 0;                    // DSR sensitivity 
    dcb.fTXContinueOnXoff = 0;                  // XOFF continues Tx 
    dcb.fErrorChar = 0;                         // enable error replacement 
    dcb.fNull = 0;                              // enable null stripping 
    dcb.fAbortOnError = 1;                      // abort reads/writes on error 

    if(m_nFlowControl & FC_XONXOFF)
    {
        dcb.fOutxCtsFlow = 0; 
        dcb.fRtsControl = RTS_CONTROL_DISABLE;
        dcb.fOutxDsrFlow = 0;
        dcb.fDtrControl = DTR_CONTROL_DISABLE;
    }
    else
    {
        dcb.fOutxCtsFlow = (m_nFlowControl & FC_RTSCTS) != 0; // CTS output flow control 
        dcb.fRtsControl =  (m_nFlowControl & FC_RTSCTS) ? RTS_CONTROL_HANDSHAKE : RTS_CONTROL_ENABLE;
        dcb.fOutxDsrFlow = (m_nFlowControl & FC_DTRDSR) != 0; // DSR output flow control
        dcb.fDtrControl =  (m_nFlowControl & FC_DTRDSR) ? DTR_CONTROL_HANDSHAKE : DTR_CONTROL_ENABLE;
    }

    dcb.fInX = dcb.fOutX = (m_nFlowControl & FC_XONXOFF) != 0;
    dcb.XonChar  = m_cXon;
    dcb.XoffChar = m_cXoff;
    dcb.XonLim   = 100;
    dcb.XoffLim  = 100;
    
    dcb.BaudRate = m_nBaudRate;                 // current baud rate 
    dcb.ByteSize = unsigned char(m_nBits);      // number of bits/byte, 4-8 
    dcb.Parity   = unsigned char(m_nParity);    // 0-4=no,odd,even,mark,space 
    dcb.StopBits = unsigned char(m_nStop);      // 0,1,2 = 1, 1.5, 2 
    if(!SetCommState(m_hPort, &dcb))
        return -1; 

    int br = (m_nBaudRate < 7200) ? 7200 : m_nBaudRate;
    br = int(float(br)/(m_nBits+m_nStop+1) + 0.5f);
    m_nByteDelay = CHAR_DELAY * 1000 / br;
    if(m_nByteDelay < 1)
        m_nByteDelay = 1;

    m_cbRead.Erase();
    
    DBGF("CommLight", "Setup OK! [%08X], Baud=%d, Bits=%d, Stop=%d, Parity=%d, Flow=%X",
         this, m_nBaudRate, m_nBits, m_nStop, m_nParity, m_nFlowControl);

    return 1;
}
/*######################################################################*/
/*Interface Interface Interface Interface Interface Interface Interface */
unsigned __stdcall  DCommLight::AddRef()  
{
    return ++m_uRefCount;
}
/*----------------------------------------------------------------------*/
unsigned __stdcall  DCommLight::Release() 
{
    if(--m_uRefCount == 0) 
    { 
        delete this; 
        return 0; 
    }
    return m_uRefCount;
}
/*======================================================================*/
void __stdcall      DCommLight::Close()
{
    MInterlocked<int>::CompareExchange((int&)m_nConnected, 0, 1);
    if(m_hPort != INVALID_HANDLE_VALUE)
    {
        SetCommMask(m_hPort, 0);
        
        if(WaitForSingleObject(m_hReadThread, 1000) != WAIT_OBJECT_0) 
            TerminateThread(m_hReadThread, 0xFFFFFFFF);
        m_hReadThread = 0;
    
        EscapeCommFunction(m_hPort, CLRDTR);
        PurgeComm(m_hPort, PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR);
        CloseHandle(m_hPort); 
        m_hPort = INVALID_HANDLE_VALUE;
        DBGF("CommLight", "Close [%08X]", this);
    }

    if(m_osWrite.hEvent) 
        CloseHandle(m_osWrite.hEvent);
    if(m_osRead.hEvent)  
        CloseHandle(m_osRead.hEvent);
    memset(&m_osWrite, 0, sizeof(m_osWrite));
    memset(&m_osRead,  0, sizeof(m_osRead));
    m_pName[0] = 0;
    m_nLastError = 0;

    m_cbRead.Erase();
}
/*----------------------------------------------------------------------*/
int __stdcall       DCommLight::Open(int pNum, int baud, 
                                     int bit, int stop, int parity, int flow)
{
    DBGF("CommLight", "Open [%08X], COM%d, Baud=%d, Bits=%d, Stop=%d, Parity=%d, Flow=%d",
         this, pNum, baud, bit, stop, parity, flow);
    
    m_nLastError = 0;
    if(m_nConnected || m_hPort != INVALID_HANDLE_VALUE) 
        return 0;

    wsprintf(m_pName, "\\\\.\\COM%d", pNum);
    m_hPort = CreateFile(m_pName, 
                         GENERIC_READ|GENERIC_WRITE, 
                         0, 0,
                         OPEN_EXISTING, 
                         FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,
                         0);
    if(m_hPort == INVALID_HANDLE_VALUE) 
        return -1;

    SetCommMask(m_hPort, 0);

    COMMPROP prop; prop.wPacketLength = sizeof(prop);
    GetCommProperties(m_hPort, &prop); 
    if(!(prop.dwProvSubType      & PST_RS232)  ||
       !(prop.dwProvCapabilities & PCF_RTSCTS) ||
       !(prop.dwProvCapabilities & PCF_DTRDSR)) 
    {
        Close();
        return -1;
    }

    for(;;)
    {
        int nTXsize = (prop.dwMaxTxQueue) ? prop.dwMaxTxQueue : 0x10000;
        int nRXsize = (prop.dwMaxRxQueue) ? prop.dwMaxRxQueue : 0x10000;

        m_osWrite.hEvent = CreateEvent(0, TRUE, FALSE, 0);
        m_osRead.hEvent  = CreateEvent(0, TRUE, FALSE, 0);
        if(!m_osWrite.hEvent || !m_osRead.hEvent) 
            break;

        if(!SetupComm(m_hPort, nTXsize, nRXsize)) 
        {
            nTXsize = prop.dwMaxTxQueue;
            nRXsize = prop.dwMaxRxQueue;
            if(!SetupComm(m_hPort, nTXsize, nRXsize)) 
                break;
        }
        PurgeComm(m_hPort, PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR);
        FlushFileBuffers(m_hPort);
        DWORD ef; 
        ClearCommError(m_hPort, &ef, 0);
        ClearCommBreak(m_hPort);
        SetCommMask(m_hPort, EV_RXCHAR|EV_ERR);

        COMMTIMEOUTS CommTimeOuts;
        CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF;
        CommTimeOuts.ReadTotalTimeoutMultiplier = 0;
        CommTimeOuts.ReadTotalTimeoutConstant = 1000;
        CommTimeOuts.WriteTotalTimeoutMultiplier = 0;
        CommTimeOuts.WriteTotalTimeoutConstant = 0;
        SetCommTimeouts(m_hPort, &CommTimeOuts);

        if(Setup(baud, bit, stop, parity, flow) <= 0) 
            break;
        DWORD status;
        if(!GetCommModemStatus(m_hPort, &status)) 
            break;
        EscapeCommFunction(m_hPort, SETDTR);

        m_nConnected = 1;

        m_hReadThread = CreateThread(0, 0,
                                     LPTHREAD_START_ROUTINE(CommReadProc),
                                     LPVOID(this),
                                     CREATE_SUSPENDED, 
                                     &m_dwReadThreadID);
        if(m_hReadThread == 0)
            break;
        SetThreadPriority(m_hReadThread, THREAD_PRIORITY_NORMAL);
        if(m_hReadThread)
            ResumeThread(m_hReadThread);

        DBGF("CommLight", "Open Connected! [%08X], COM%d", this, pNum);
    
        return 1;
    }
    int ecode = GetLastError();
    Close(); 
    SetLastError(ecode);
    return -1; 
}
/*----------------------------------------------------------------------*/
unsigned __stdcall  DCommLight::Look(int code)
{
    switch(code)
    {
    case    LOOK_BAUD:      return m_nBaudRate;
    case    LOOK_STOP:      return m_nStop;
    case    LOOK_PARITY:    return m_nParity;
    case    LOOK_FLOW:      return m_nFlowControl;
    case    LOOK_SIZE:      return m_nBits;
    case    LOOK_PORT:      return unsigned(&m_pName);
    case    LOOK_CONNECT:   return m_nConnected;
    case    LOOK_LASTERROR: 
        {
            int ecode = m_nLastError; m_nLastError = 0;
            return ecode;
        }
    case    LOOK_LASTERRORSTRING: 
        {
            int ecode = m_nLastError; m_nLastError = 0;
            switch(ecode)
            {
            case CL_WRITE_ERROR:            
                return unsigned("Write error");
            case CL_READ_ERROR:             
                return unsigned("Read error");
            case CL_READ_ERROR_RXPARITY:
                return unsigned("The hardware detected a parity error.");
            case CL_READ_ERROR_FRAME: 
                return unsigned("The hardware detected a framing error.");
            case CL_READ_ERROR_OVERRUN:
                return unsigned("A character-buffer overrun has occurred. The next character is lost.");
            }
            return unsigned("No error");
        }
    }
    return 0;
}
/*======================================================================*/
void __stdcall      DCommLight::Clear()
{
#ifdef  FULL_DEBUG
    DBGF("CommLight", "Clear [%08X]", this);
#endif

    if(m_hPort == INVALID_HANDLE_VALUE)
        return;
    
    EnterCriticalSection(&m_cs);
    PurgeComm(m_hPort, PURGE_TXCLEAR|PURGE_RXCLEAR);
    DWORD ef; 
    ClearCommError(m_hPort, &ef, 0);
    m_cbRead.Erase();
    LeaveCriticalSection(&m_cs);
}
/*----------------------------------------------------------------------*/
unsigned __stdcall  DCommLight::Get()
{
    if(m_hPort == INVALID_HANDLE_VALUE)
        return 0;

    EnterCriticalSection(&m_cs);
    unsigned ret = m_cbRead.Get();
    LeaveCriticalSection(&m_cs);
    return ret;
}
/*----------------------------------------------------------------------*/
unsigned __stdcall  DCommLight::Test(int i)
{
    if(m_hPort == INVALID_HANDLE_VALUE)
        return 0;

    EnterCriticalSection(&m_cs);
    unsigned ret = m_cbRead.Test(i);
    LeaveCriticalSection(&m_cs);
    return ret;
}
/*----------------------------------------------------------------------*/
int __stdcall       DCommLight::Write(char* buff, int cnt, unsigned tout)
{
    if(m_hPort == INVALID_HANDLE_VALUE || buff == 0 || cnt <= 0)
        return 0;

    int wcnt = cnt;
    int ws = WriteFile(m_hPort, buff, cnt, (DWORD*)&wcnt, &m_osWrite);
    if(!ws)
    {
        if(GetLastError() == ERROR_IO_PENDING)
        {
            unsigned delay = m_nByteDelay * cnt / 100 + 1;
            Sleep(delay);
            while(m_nConnected && !GetOverlappedResult(m_hPort, &m_osWrite, (DWORD*)&wcnt, FALSE))
            {
                if(GetLastError() == ERROR_IO_INCOMPLETE) 
                { 
                    Sleep(delay);
                    if((tout -= delay) > 0)
                        continue;
                    wcnt = 0;
                }
                ErrorHandler(0);
                break;
            }
        }
        else
        {
            ErrorHandler(0);
            return 0;
        }
    }

#ifdef  FULL_DEBUG
    DBGF("CommLight", "Write [%08X], Cnt(Send=%d/Total=%d)", this, wcnt, cnt);
#endif
    return wcnt;
}
/*----------------------------------------------------------------------*/
int __stdcall       DCommLight::Read(char* buff, int cnt, unsigned tout)
{
    if(m_hPort == INVALID_HANDLE_VALUE || buff == 0 || cnt <= 0)
        return 0;

    unsigned tout0 = GetTickCount();
    for(int rcnt = 0; rcnt < cnt && GetTickCount()-tout0 < tout;)
    {
        unsigned c = Get();
        if(c == 0)
            Sleep(0);
        else
            buff[rcnt++] = char(c & 0xFF);
    }

#ifdef  FULL_DEBUG
    DBGF("CommLight", "READ [%08X], Cnt(Read=%d/Total=%d)", this, rcnt, cnt);
#endif
    return rcnt;
}
/*======================================================================*/
void __stdcall      DCommLight::EscapeFunction(unsigned fun)
{
    if(m_hPort != INVALID_HANDLE_VALUE)
    {
        EscapeCommFunction(m_hPort, fun);

#ifdef  FULL_DEBUG
        DBGF("CommLight", "EscapeFunction [%08X], Function=%08X", this, fun);
#endif
    }
}
/*----------------------------------------------------------------------*/
unsigned __stdcall  DCommLight::GetModemStatus()
{
    if(m_hPort == INVALID_HANDLE_VALUE) 
        return 0;

    unsigned dwStat;
    if(!GetCommModemStatus(m_hPort, (DWORD*)&dwStat)) 
        return (unsigned)-1;

#ifdef  FULL_DEBUG
    DBGF("CommLight", "GetModemStatus OK! [%08X], Status=%08X", this, dwStat);
#endif
    return dwStat;
}
/*######################################################################*/
ICommLight* __stdcall   ICommLightCreate()
{
    ICommLight* ComDevice = new DCommLight;
    if(ComDevice)
        ComDevice->AddRef();
    return ComDevice;
}
/*######################################################################*/
