/*######################################################################*/
//  Revision Date       04_10_2007
/*######################################################################*/
#include    "Windows.h"
#include    "ConfigFile.h"
/*######################################################################*/
static unsigned char XlatTable[] =
"                                "
"           + -. 0123456789;;    "
" ABCDEFGHIJKLMNOPQRSTUVWXYZ    _"
" ABCDEFGHIJKLMNOPQRSTUVWXYZ     "
"                                "
"                                "
"                                "
"                                ";
/*======================================================================*/
ConfigFile::ConfigFile(char const* path, char* buff, int size)
{
    m_str   = 0;
    m_nPos  = 0;
    m_nSize = 0;
    HANDLE hFile = INVALID_HANDLE_VALUE;

    try
    {
        if(buff != 0 && size > 0)
        {
            m_str = new unsigned char[size+1];
            if(m_str == 0)
                return;
            m_nSize = size;
        }
        else
        {
            hFile = ::CreateFile(path, GENERIC_READ, FILE_SHARE_READ,
                                 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
            if(hFile == INVALID_HANDLE_VALUE) 
                return;

            size = ::GetFileSize(hFile, 0);
            if(size <= 0 || size > 128*1024)
            {
                ::CloseHandle(hFile);
                return;
            }
        
            m_str = new unsigned char[size+1];
            if(m_str == 0)
            {
                ::CloseHandle(hFile);
                return;
            }
            
            DWORD dw;
            ::ReadFile(hFile, m_str, size, &dw, 0);
            ::CloseHandle(hFile);

            if(dw != DWORD(size))
            {
                delete[] m_str; 
                m_str = 0;
                return;
            }
            m_nSize = size;
        }
    
        for(int i = 0, ml = 0, fa = 0, fc = 0; i < m_nSize; ++i)
        {
            unsigned char c = m_str[i];
            if(fa == 0 && fc == 0 && (c == '\\' || c == '/')) 
                ++ml;
            if(c == '\n')
            {
                if(ml == 1) 
                    m_str[i] = ' ';
                else        
                    m_str[i] = 0;
                fa = fc = ml = 0;
            }
            else 
            {
                if(fa && c != '"')
                {
                    m_str[i] = c;
                }
                else
                {
                    m_str[i] = XlatTable[c];
                    if(c == '"') 
                        fa ^= 1;
                    if(m_str[i] == ';') 
                        fc = 1;
                    if(fc) 
                        m_str[i] = ' ';
                }
            }
        }
        m_str[m_nSize] = 0;
    }
    catch(...)
    {
        m_str   = 0;
        m_nPos  = 0;
        m_nSize = 0;
        if(hFile != INVALID_HANDLE_VALUE) 
            ::CloseHandle(hFile);
    }
}
/*----------------------------------------------------------------------*/
ConfigFile::~ConfigFile()
{
    if(m_str) 
        delete[] m_str;
}
/*======================================================================*/
int     ConfigFile::GetKey(int index, char* key, int size)
{
    if(m_str == 0) 
        return -1;
    for(int n0 = 0; n0 < m_nSize;)
    {
        int n1;
        for(n1 = n0; n1 < m_nSize && m_str[n1]; ++n1);
        if(--index < 0)
        {
            int i;
            for(i = n0; i < n1; ++i) 
                if(m_str[i] > ' ') 
                    break;
            for(; i < n1 && size > 0 && m_str[i] > ' '; ++i, --size) 
                *key++ = m_str[i];
            return 1;
        }
        n0 = n1 + 1;
    }
    return 0;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyStr(char* ikey, int& nMax, int& nPos)
{
    if(ikey && *ikey)
    {
        char key[256];

        int i = 0, ki = 0;
        for(; i < sizeof(key) && ikey[i]; ++i) 
            key[i] = XlatTable[ikey[i] & 0x7F];
        for(; key[ki] && key[ki] == ' '; ++ki);
        key[i] = 0;
        for(int n0 = 0, pos = 0; n0 < m_nSize; ++pos)
        {
            int n1;
            for(n1 = n0; n1 < m_nSize && m_str[n1]; ++n1);
            for(int i = n0; i < n1; ++i)
            {
                if(key[ki] == m_str[i])
                {
                    if(i > 0 && m_str[i-1] > ' ') 
                        continue;
                    int j = ki+1, k = 1;
                    for(; key[j] && (i+k < n1);)
                    {
                        if(m_str[i+k] == ' ') 
                        { 
                            ++k; 
                            continue; 
                        }
                        if(key[j]     == ' ') 
                        { 
                            ++j;
                            continue; 
                        }
                        if(key[j] != m_str[i+k])
                            break;
                        ++j;
                        ++k;
                    }
                    if(key[j] == 0 && m_str[i+k] <= ' ')
                    {
                        nMax = n1;
                        nPos = i+k;
                        m_nPos = pos;
                        return 1;
                    }
                }
            }
            n0 = n1 + 1;
        }
    }
    else
    {
        for(int n0 = 0, pos = m_nPos; n0 < m_nSize;)
        {
            int n1;
            for(n1 = n0; n1 < m_nSize && m_str[n1]; ++n1);
            for(int i = n0; i < n1; ++i) 
                if( m_str[i] > ' ')
                {
                    if(--pos >= 0)
                        break;

                    nMax = n1;
                    nPos = i;
                    m_nPos++;
                    return 1;
                }
            n0 = n1 + 1;
        }
    }
    return 0;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::TestNumber(int max, int& n)
{
    int ret = 0;
    int n0;
    for(n0 = n; n0 < max && m_str[n0] <= ' '; ++n0);
    if(n0 < max)
    {
        ret = 1;
        if((n0+1 < max) && m_str[n0] == '0' && m_str[n0+1] == 'X')
        {
            for(int i = n0+2; i < max && m_str[i] > ' '; ++i)
            {
                char c = m_str[i];
                if((c < '0' || c > '9') && (c < 'A' || c > 'F'))
                {
                    ret = 0; 
                    break;
                }
            }
        }
        else
        {
            if(m_str[n0] == '-' || m_str[n0] == '+') 
                ++n0;
            for(int floatf = 0, expf = 0, i = n0; i < max && m_str[i] > ' '; ++i)
            {
                char c = m_str[i];
                if(c == '.') 
                    ++floatf;
                else
                if(c == 'D' || c == 'E') 
                    ++expf;
                else
                if(c < '0' || c > '9') 
                {
                    if(expf == 1 && (c == '-' || c == '+'))
                        continue;
                    if(m_str[i+1] > ' ' || (c != 'K' && c != 'M' && c != 'T' && c != 'L'))
                    {
                        ret = 0; 
                        break;
                    }
                }
                if((floatf | expf) > 1)
                {
                    ret = 0; 
                    break;
                }
            }
        }
    }
    for(; n0 < max && m_str[n0] > ' '; ++n0);
    n = n0;
    return ret;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetNumber(int max, int& n, int* ival, double* dval, float* fval)
{
    int ret = 0;
    int n0;
    for(n0 = n; n0 < max && m_str[n0] <= ' '; ++n0);
    if(n0 < max)
    {
        ret = 1;
        if((n0+1 < max) && m_str[n0] == '0' && m_str[n0+1] == 'X')
        {
            __int64 val = 0;
            for(int i = n0+2; i < max && m_str[i] > ' '; ++i)
            {
                val <<= 4;
                char c = m_str[i];
                if(c >= '0' && c <= '9')
                    val += c - '0';
                else
                if(c >= 'A' && c <= 'F')
                    val += c - 'A' + 10;
                else 
                {
                    ret = 0; 
                    break;
                }
            }
            if(ret)
            {
                if(ival) 
                    *ival = int   (val);
                if(dval) 
                    *dval = double(val);
                if(fval) 
                    *fval = float (val);
            }
        }
        else
        {
            char sign = 0, mk = 0;
            if(m_str[n0] == '-') 
                sign = 1, ++n0;
            else
            if(m_str[n0] == '+')
                ++n0;
            int floatf = 0, expf = 0, n1 = n0;
            for(; n1 < max && m_str[n1] > ' '; ++n1) 
            {
                char c = m_str[n1];
                if(c == '.') 
                    ++floatf;
                else
                if(c == 'D' || c == 'E')
                    ++expf;
                else
                if(c < '0' || c > '9')
                {
                    if(expf == 1 && (c == '-' || c == '+'))
                        continue;
                    if(m_str[n1+1] <= ' ' && (c == 'K' || c == 'M' || c == 'T' || c == 'L'))
                        mk = c;
                    else
                        ret = 0; 
                    break;
                }
            }
            if(ret)
            {
                if((floatf | expf) == 1)
                {
                    char str[100];
                    int i;
                    for(i = n0; i < n1 && (i-n0) < sizeof(str); ++i)
                        str[i-n0] = m_str[i];
                    str[i-n0] = 0;
                    
                    double val = atof(str);
                    if(mk == 'K') 
                        val *= 1024.0;
                    else
                    if(mk == 'M') 
                        val *= 1024.0*1024.0;
                    else
                    if(mk == 'T') 
                        val *= 1000.0;
                    else
                    if(mk == 'L') 
                        val *= 1000.0*1000.0;

                    if(sign)
                        val = -val;
                    if(ival)
                        *ival = int(val + 0.5);
                    if(dval)
                        *dval = val;
                    if(fval)
                        *fval = float(val);
                }
                else
                if(expf == 0 && floatf == 0)
                {
                    __int64 val = 0;
                    for(int i = n0; i < n1; ++i)
                    {
                        val *= 10;
                        char c = m_str[i];
                        if(c >= '0' && c <= '9')
                            val += c - '0';
                        else 
                        {
                            ret = 0; 
                            break;
                        }
                    }
                    if(ret) 
                    {
                        if(mk == 'K')
                            val *= 1024;
                        else
                        if(mk == 'M') 
                            val *= 1024*1024;
                        else
                        if(mk == 'T')
                            val *= 1000;
                        else
                        if(mk == 'L') 
                            val *= 1000*1000;

                        if(sign)
                            val = -val;
                        if(ival)
                            *ival = int   (val);
                        if(dval)
                            *dval = double(val);
                        if(fval)
                            *fval = float (val);
                    }
                }
                else ret = 0; 
            }
        }
    }
    for(; n0 < max && m_str[n0] > ' '; ++n0);
    n = n0;
    return ret;
}
/*======================================================================*/
int     ConfigFile::TestKey(char* key)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    int pos = m_nPos;
    int ret = GetKeyStr(key, nMax, nPos);
    m_nPos = pos;
    return ret;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::TestKeyValue(char* key)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    int pos = m_nPos;
    if(GetKeyStr(key, nMax, nPos))
    {
        m_nPos = pos;
        int i;
        for(i = 0; TestNumber(nMax, nPos) > 0; ++i);
        return i;
    }
    m_nPos = pos;
    return 0;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValue(char* key, char* val, int size, char* defval)
{
    if(m_str == 0) 
        return -1;
    if(val == 0 || size <= 0) 
        return 0;
    int nMax, nPos;
    if(GetKeyStr(key, nMax, nPos) == 0) 
    {
        if(defval && *defval)
            strcpy(val, defval);
        return 0;
    }
    int i, j;
    for(i = nPos; i < nMax && m_str[i] <= ' '; ++i);
    for(j = 0; j+i < nMax && j < size-1; ++j) 
        val[j] = m_str[j+i];
    for(val[j--] = 0; j > 0 && val[j] == ' '; --j)
        val[j] = 0;
    return 1;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValue(char* key, int& val, int defval)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    if((GetKeyStr(key, nMax, nPos)        == 0) ||
       (GetNumber(nMax, nPos, &val, 0, 0) <= 0))
    {
        if(defval != 0) 
            val = defval;
        return 0;
    }
    return 1;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValue(char* key, short& val, short defval)
{
    if(m_str == 0) 
        return -1;
    int tmp;
    if(GetKeyValue(key, tmp, int(defval)) > 0)
    {
        val = short(tmp & 0xFFFF);
        return 1;
    }
    if(defval)
        val = defval;
    return 0;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValue(char* key, double& val, double defval)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    if((GetKeyStr(key, nMax, nPos)        == 0) ||
       (GetNumber(nMax, nPos, 0, &val, 0) <= 0))
    {
        if(defval != 0.0) 
            val = defval;
        return 0;
    }
    return 1;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValue(char* key, float& val, float defval)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    if((GetKeyStr(key, nMax, nPos)        == 0) ||
       (GetNumber(nMax, nPos, 0, 0, &val) <= 0))
    {
        if(defval != 0.0f)
            val = defval;
        return 0;
    }
    return 1;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValueN(char* key, int* val, int size)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    if(GetKeyStr(key, nMax, nPos) == 0) 
        return 0;
    int i;
    for(i = 0; i < size; ++i)
        if(GetNumber(nMax, nPos, &val[i], 0, 0) <= 0) 
            break;
    return i;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValueN(char* key, double* val, int size)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    if(GetKeyStr(key, nMax, nPos) == 0) 
        return 0;
    int i;
    for(i = 0; i < size; ++i)
        if(GetNumber(nMax, nPos, 0, &val[i], 0) <= 0) 
            break;
    return i;
}
/*----------------------------------------------------------------------*/
int     ConfigFile::GetKeyValueN(char* key, float* val, int size)
{
    if(m_str == 0) 
        return -1;
    int nMax, nPos;
    if(GetKeyStr(key, nMax, nPos) == 0) 
        return 0;
    int i;
    for(i = 0; i < size; ++i)
        if(GetNumber(nMax, nPos, 0, 0, &val[i]) <= 0) 
            break;
    return i;
}
/*######################################################################*/
