#include    <Windows.h>
#include    "MTypes.h"
#include    "MBrush.h"
#include    "MToolTip.h"

#include    "MWC.h"
/*######################################################################*/
#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED           0x00080000
#endif
#ifndef LWA_COLORKEY
#define LWA_COLORKEY            0x00000001
#endif
#ifndef LWA_ALPHA
#define LWA_ALPHA               0x00000002
#endif

#define HELP_RATIO              2

#define HIDE_VALUE              200
typedef BOOL (__stdcall* SetLayeredWindow)(HWND, COLORREF, BYTE, DWORD);
/*######################################################################*/
static int cpg[] = {  0,  0,  1,  6,
                      2, 12,  3, 16,
                      4, 19,  5, 21,
                      7, 25, 11, 28,
                     14, 29, 22, 30,
                     31, 30, 34, 29,
                     36, 28, 38, 25,
                     39, 23, 40, 20,
                     40, 18, 39, 15,
                     38, 13, 36, 11,
                     34,  9, 30,  7,
                     24,  6, 13,  6,
                      6,  5,  2,  3,
                      0,  0 };

static  RECT rx =   { 9, 26,  34, 9 }; 
/*======================================================================*/
class   HWindow : public HelpWindow, public MWindow
{
public:
    void    __stdcall   Close();
    void    __stdcall   Display(HWND hp, const TCHAR* str, int x, int y, int tout);

private:
    class   HClass : public MClass
    {
    public:
        void    Register(WNDCLASSEX& wc)
        {
            wc.style         = WS_EX_TRANSPARENT;
            wc.hIcon         = 0;
            wc.hCursor       = LoadCursor(0, IDC_HELP);
            wc.hbrBackground = HBRUSH(GetStockObject(NULL_BRUSH));
            wc.lpszMenuName  = NULL;
        }
    } m_wc;

    bool                m_run;
    SetLayeredWindow    m_pf;
    int                 m_nf;
    HRGN                m_hRgnW, m_hRgnB;
    RECT                m_rcIn;
    int                 m_nDirection;
    int                 m_nTout;
    const char*         m_str;
    MBrush              m_brush;
    int*                m_pPolygonRegion;
    int                 m_nPolygonSize;
    RECT                m_RectInPolygon;
    int*                m_pPt;
    int                 m_nMaxX, m_nMaxY;

    void    MakeRgn();
    void    Display(HWND, const TCHAR*, int, int, int, int);
    int     GetTextRect(HWND, const char* str, RECT& rc);

public:      
    HWindow(int idr, int* pPolygonRegion, int PolygonSize, RECT* pRectInPolygon) : MWindow() 
    { 
        m_run = false; 
        m_pf  = 0;
        m_nf  = 0;
        m_hRgnW = m_hRgnB = 0; 
        m_nTout = 4000; 
        if(idr) 
            m_brush.SetImage(GetModuleHandle(0), MAKEINTRESOURCE(idr));
        if(pPolygonRegion != 0 && PolygonSize > 3 && pRectInPolygon != 0)
        {
            m_nPolygonSize   = PolygonSize * 2;
            m_RectInPolygon  = *pRectInPolygon;
            m_pPolygonRegion = new int[m_nPolygonSize];
            for(int i = 0; i < m_nPolygonSize; ++i)
                m_pPolygonRegion[i] = pPolygonRegion[i];
        }
        else
        {
            m_pPolygonRegion = cpg;
            m_nPolygonSize   = DIM(cpg);
            m_RectInPolygon  = rx;
        }
    
        m_nDirection = 0;
        m_nMaxX = 0;
        m_nMaxY = 0;
        for(int i = 0; i < m_nPolygonSize; i += 2)
        {
            if(m_nMaxX < m_pPolygonRegion[i+0])
                m_nMaxX = m_pPolygonRegion[i+0];
            if(m_nMaxY < m_pPolygonRegion[i+1])
                m_nMaxY = m_pPolygonRegion[i+1];
        }
        m_nMaxX += HELP_RATIO;
        m_nMaxY += 1;

        m_pPt = new int[m_nPolygonSize];
    }

    ~HWindow()
    {
        if(m_hRgnW) 
            DeleteObject(m_hRgnW);
        if(m_hRgnB)
            DeleteObject(m_hRgnB);
        if(m_pPt)
            delete[] m_pPt;
        if(m_pPolygonRegion != 0 && m_pPolygonRegion != cpg)
            delete[] m_pPolygonRegion;
    }

private:
    int     Msg_Create();
    int     Msg_Kill();
    int     Msg_NCHITTEST();
    
    void    OnDestroy();
    void    OnPaint(HDC, RECT&);

D_MESSAGE_MAP()
};
/*======================================================================*/
B_MESSAGE_MAP(HWindow)
MSG_ENTRY(HWindow,  WM_NCHITTEST,   Msg_NCHITTEST)
MSG_ENTRY(HWindow,  WM_CREATE,      Msg_Create)
MSG_ENTRY(HWindow,  WM_LBUTTONDOWN, Msg_Kill)
MSG_ENTRY(HWindow,  WM_TIMER,       Msg_Kill)
E_MESSAGE_MAP()
/*######################################################################*/
void    HWindow::MakeRgn()
{
#define hX(b)   (1+     ((b) * mx >> 16))
#define hY(b)   (dy+1 - ((b) * my >> 16))

    if(m_hRgnW)
        DeleteObject(m_hRgnW);
    if(m_hRgnB) 
        DeleteObject(m_hRgnB);

    RECT rc; GetWindowRect(hWnd, &rc);
    int dx = rc.right - rc.left;
    int dy = rc.bottom - rc.top;
    int mx = dx * 0x10000 / m_nMaxX; 
    int my = dy * 0x10000 / m_nMaxY;
    if(m_nDirection & 1)
    {
        m_rcIn.left   = dx - hX(rx.right);  
        m_rcIn.right  = dx - hX(rx.left); 
    }
    else
    {
        m_rcIn.left   = hX(rx.left);  
        m_rcIn.right  = hX(rx.right); 
    }
    if(m_nDirection & 2)
    {
        m_rcIn.top    = dy - hY(rx.bottom);
        m_rcIn.bottom = dy - hY(rx.top);
    }
    else
    {
        m_rcIn.top    = hY(rx.top);
        m_rcIn.bottom = hY(rx.bottom);
    }
    for(int i = 0; i < m_nPolygonSize; i += 2)
    {
        m_pPt[i+0] = hX(m_pPolygonRegion[i+0]);     
        if(m_nDirection & 1)
            m_pPt[i+0] = dx - m_pPt[i+0];
        m_pPt[i+1] = hY(m_pPolygonRegion[i+1]);
        if(m_nDirection & 2)
            m_pPt[i+1] = dy - m_pPt[i+1];
    }
    m_hRgnW = CreatePolygonRgn(LPPOINT(m_pPt), m_nPolygonSize/2, WINDING);
    m_hRgnB = CreateRectRgn(0, 0, 0, 0);
    CombineRgn(m_hRgnB, m_hRgnW, m_hRgnW, RGN_COPY);

    SetWindowRgn(hWnd, m_hRgnW, true); 
    
    HMODULE hLib = HMODULE(::LoadLibrary("User32.dll"));
    if(hLib != 0) 
    {
        m_pf = (SetLayeredWindow)::GetProcAddress(hLib, "SetLayeredWindowAttributes");
        if(m_pf != 0)
            m_pf(hWnd, 0, m_nf = HIDE_VALUE, LWA_ALPHA);
    }
}
/*----------------------------------------------------------------------*/
void    HWindow::OnPaint(HDC hdc, RECT& rc)
{
    SetBkMode(hdc, TRANSPARENT);
    SetTextColor(hdc, RGB(0,0,0));
    SelectObject(hdc, GetStockObject(NULL_PEN));
    SelectObject(hdc, GetSysColorBrush(COLOR_INFOBK));
    SelectObject(hdc, GetStockObject(SYSTEM_FONT));
    SetPolyFillMode(hdc, WINDING);
    if(m_brush.OK()) 
        m_brush.FillRect(hdc, &rc);
    else            
        Polygon(hdc, LPPOINT(m_pPt), m_nPolygonSize/2);
    FrameRgn(hdc, m_hRgnB, HBRUSH(GetStockObject(DKGRAY_BRUSH)), 1, 1); 
    DrawText(hdc, m_str, -1, &m_rcIn, DT_CENTER|DT_WORDBREAK);
}
/*----------------------------------------------------------------------*/
void    HWindow::OnDestroy()   
{ 
    KillTimer(hWnd, 1);
    m_run = false;
    hWnd = 0;
}
/*----------------------------------------------------------------------*/
int     HWindow::Msg_Create()
{
    MakeRgn();
    SetWindowText(hWnd, m_str);
    m_run = true;
    SetTimer(hWnd, 1, m_nTout, 0);
    return 1;
}
/*----------------------------------------------------------------------*/
int     HWindow::Msg_Kill()
{
    if(m_pf != 0)
    {
        m_pf(hWnd, 0, m_nf, LWA_ALPHA);
        if(m_nf == HIDE_VALUE)
            SetTimer(hWnd, 1, 40, 0);
        m_nf -= 5;
        if(m_nf < 0)
            m_nf = 0;
    }
    if(m_nf == 0)
    {
        KillTimer(hWnd, 1);
        DestroyWindow(hWnd);
    }
    return 0;
}
/*----------------------------------------------------------------------*/
int     HWindow::Msg_NCHITTEST()
{
    return HTTRANSPARENT;
}
/*======================================================================*/
int     GetWordCount(const char* s)
{
    if(s == 0 || *s == 0)
        return 0;
    for(int i=0; ;)
    {
        while(*s && *s == ' ') ++s; 
        if(*s != 0) ++i;
        while(*s && *s != ' ') ++s; 
        if(*s == 0) break;
    }
    return i;
}
/*----------------------------------------------------------------------*/
int     HWindow::GetTextRect(HWND hWnd, const char* str, RECT& rc)
{
    rc.left = rc.top = rc.right = rc.bottom = 0;
    int n = GetWordCount(str);
    if(n == 0) 
        return 0;

    HDC hdc = GetDC(hWnd);
    HFONT hof = HFONT(SelectObject(hdc, GetStockObject(SYSTEM_FONT)));
    int om = SetMapMode(hdc, MM_TEXT);
    
    const char* s = str; 
    SIZE siz;
    for(int i = 0, xmax =- 1; i < n; ++i)
    {
        while(*s && *s == ' ') ++s; 
        for(int j = 0; s[j] && s[j] != ' '; ++j);
        GetTextExtentPoint32(hdc, s, j+1, &siz);
        if(xmax < siz.cx) 
            xmax = siz.cx;
        s += j;
    }
    GetTextExtentPoint32(hdc, "000", 3, &siz);
    xmax += siz.cx;
    if(i > 2)
        xmax *= HELP_RATIO;

    rc.right = xmax;
    DrawText(hdc, str, -1, &rc, DT_CENTER|DT_CALCRECT|DT_WORDBREAK);
    rc.right  = rc.right  * m_nMaxX / (m_RectInPolygon.right-m_RectInPolygon.left);
    rc.bottom = rc.bottom * m_nMaxY / (m_RectInPolygon.top-m_RectInPolygon.bottom);

    SetMapMode(hdc, om);
    SelectObject(hdc, hof);
    ReleaseDC(hWnd, hdc);
    return n;
}
/*----------------------------------------------------------------------*/
void    HWindow::Display(HWND hp, const TCHAR* str, int x, int y, int wid, int hei)
{
    if(*m_wc.GetClassName() == 0)
    {
        m_wc.Initialize("WindowsHelperClass");
        m_wc.RegisterClass();
    }
    if(m_run && hWnd) 
    {
        DestroyWindow(hWnd); 
        m_run = false;
    }
    m_str = str;
    Create(m_wc, str, WS_POPUP|WS_VISIBLE, WS_EX_TOPMOST|WS_EX_LAYERED, x, y, wid, hei, hp, 0);
   
    if(hWnd)
        m_run = true;
}
/*######################################################################*/
void    HWindow::Close()
{
    if(hWnd) 
        DestroyWindow(hWnd);
}
/*----------------------------------------------------------------------*/
void    HWindow::Display(HWND hp, const TCHAR* str, int x, int y, int tout)
{
    RECT rc; 
    int n = GetTextRect(hp, str, rc);
    if(n <= 0) 
        return;
    int wid = rc.right - rc.left + 1;
    int hei = rc.bottom - rc.top + 1;
    int xm = GetSystemMetrics(SM_CXSCREEN) - wid-1;
    int ym = GetSystemMetrics(SM_CYSCREEN) - hei-1;
    POINT p; 
    p.x = x; p.y = y; 
    ClientToScreen(0, &p);
    x = p.x; 
    y = p.y - hei;

    if(x < 0)
        x = 0;
    else
    if(x > xm)
    {
        m_nDirection |= 1;
        x -= wid;
    }
    if(y < 0)
    {
        m_nDirection |= 2;
        y += hei;
    }
    else
    if(y > ym)
        y = ym;

    n /= 10; 
    if(n == 0) 
        n = 1;
    m_nTout = CutV(tout, 500, 60000) * n;
    Display(hp, str, x, y, wid, hei);
}
/*----------------------------------------------------------------------*/
HelpWindow* __stdcall HelpWindow::New(int idr, int* pPolygonRegion, int PolygonSize, RECT* pRectInPolygon)
{
    HWindow* ph = new HWindow(idr, pPolygonRegion, PolygonSize, pRectInPolygon);
    return (HelpWindow*)ph;
}
/*######################################################################*/
