#include    <windows.h>
#include    "MBrush.h"
/*######################################################################*/
void    MBrush::SetImage(HINSTANCE hI, const TCHAR* name)
{
    if(m_hBitmap) 
        DeleteObject(m_hBitmap), m_hBitmap = 0;
    m_cx = m_cy = 0;
    HRSRC hrsrc = FindResource(hI, name, RT_BITMAP);
    if(hrsrc == 0) 
        return;
    LPSTR p = LPSTR(LockResource(LoadResource(hI, hrsrc)));
    if(p == 0) 
        return;
    LPBITMAPINFO bi = LPBITMAPINFO(p);
    int c = bi->bmiHeader.biBitCount; 
    c = (c < 1 || c > 8) ? 0 : 1<<c;
    m_cx = bi->bmiHeader.biWidth;
    m_cy = bi->bmiHeader.biHeight;
    p += bi->bmiHeader.biSize + sizeof(RGBQUAD) * c;
    HDC hDC = GetDC(0);
    m_hBitmap = CreateDIBitmap(hDC, LPBITMAPINFOHEADER(bi), CBM_INIT, p, 
                               bi, DIB_RGB_COLORS);
    ReleaseDC(0, hDC);
}
/*----------------------------------------------------------------------*/
void    MBrush::SetImage(const TCHAR* name_file)
{
    if(m_hBitmap) 
        DeleteObject(m_hBitmap), m_hBitmap = 0;
    m_cx = m_cy = 0;

    m_hBitmap = (HBITMAP)LoadImage(NULL, name_file, IMAGE_BITMAP, 0, 0,
                                   LR_DEFAULTSIZE | LR_LOADFROMFILE | LR_CREATEDIBSECTION);

    if(m_hBitmap)
    {
        BITMAP bm;
        GetObject(m_hBitmap, sizeof(BITMAP), LPSTR(&bm));
        m_cx = bm.bmWidth;
        m_cy = bm.bmHeight;
    }
}
/*----------------------------------------------------------------------*/
void    MBrush::FillRect(HDC hDC, RECT* rc, HRGN hClipRgn)
{       
    if(!hDC || !rc || !m_hBitmap) 
        return;
    BITMAP bm;
    if(!GetObject(m_hBitmap, sizeof(BITMAP), &bm)) 
        return;
    HDC bdc = CreateCompatibleDC(hDC);
    if(!bdc) 
        return;
    HBITMAP hob = HBITMAP(SelectObject(bdc, m_hBitmap));

    HRGN hrgn = hClipRgn;
    if(hrgn == 0)
        hrgn = CreateRectRgn(rc->left, rc->top, 
                             rc->right-rc->left+1, rc->bottom-rc->top+1);
    SelectClipRgn(hDC, hrgn);

    int x0 = (rc->left - m_ptOffset.x) % bm.bmWidth;
    int y0 = (rc->top  - m_ptOffset.y) % bm.bmHeight;
    for(int y = y0; y <= rc->bottom; y += bm.bmHeight)
        for(int x = x0; x <= rc->right; x += bm.bmWidth)
            BitBlt(hDC, x,y, bm.bmWidth,bm.bmHeight, bdc, 0,0, SRCCOPY);

    SelectClipRgn(hDC, 0); 
    
    if(hClipRgn != hrgn)
        DeleteObject(hrgn);
    SelectObject(bdc, hob);
    DeleteDC(bdc);
}
/*######################################################################*/
