/*######################################################################*/
#include    <Windows.h>

#include    "IOSCE.h"
#include    "DllClient.h"
/*======================================================================*/
#include    "MWC.h"
#include    "MWC.cpp"

#define     CLASS_NAME          "DRIVERVIEW"    

#define     WMMY_SETDATA_SHORT  WM_USER+77
#define     WMMY_SETDATA_FLOAT  WM_USER+78
/*######################################################################*/
class   MyWClass : public MClass
{
public:
    MyWClass() : MClass(CLASS_NAME) {}
    
    void    Register(WNDCLASSEX& wc)
    {
        wc.style         = CS_DBLCLKS;
        wc.hIcon         = 0;
        wc.hCursor       = LoadCursor(0, IDC_CROSS);
        wc.hbrBackground = HBRUSH(GetStockObject(NULL_BRUSH));
    }
};  
/*----------------------------------------------------------------------*/
MWindow*    NewWindow(LPSTR);

class   MyClassLoader
{
private:
    NewObjectMSG    m_pPrevNewObjectMSG;
    MyWClass        m_Class;
    DllClient       m_OscClient;
    IFactory*       m_OscFactory;

public:
    MyClassLoader()
    {
        m_Class.RegisterClass();
        m_OscFactory = (IFactory*)m_OscClient.LoadRegistered(CompanyName, 
                                                             ProductName, 
                                                             VersionName, 
                                                             "ZoomerE");
        m_pPrevNewObjectMSG = ::pNewObjectMSG;
        ::pNewObjectMSG = &NewWindow;
    }

    ~MyClassLoader()
    {
        ::pNewObjectMSG = m_pPrevNewObjectMSG;
    }

    IOSCE*      GetOSC()
    {
        return (IOSCE*)(m_OscFactory ? m_OscFactory->Create("IZoomerE", 0) : 0);
    }

    MWindow*    CallNextNewWindow(LPSTR str)
    {
        return m_pPrevNewObjectMSG ? m_pPrevNewObjectMSG(str) : 0;
    }
};

static 
MyClassLoader   MyLoader;
/*######################################################################*/
class   MyWindow : public MWindow
{
private:
    float*          m_pData;
    int             m_nSize, m_nCh;
    float           m_fYmi, m_fYma;
    IOSCE*          m_pOsc;

public:      
    MyWindow(IOSCE*);
    ~MyWindow();

private:
    bool    Alloc(int size);
    void    Free();

    void    OnPaint(HDC, RECT&);
    void    OnDestroy();

    int     Msg_CREATE();
    int     Msg_LBUTTONDOWN(LPARAM);
    int     Msg_LBUTTONDBLCLK(LPARAM);
    int     Msg_RBUTTONDOWN(LPARAM);
    int     Msg_MOUSEMOVE(WPARAM, LPARAM);
    int     Msg_SIZE();

    int     Msg_SETDATAshort(WPARAM, LPARAM);
    int     Msg_SETDATAfloat(WPARAM, LPARAM);

D_MESSAGE_MAP()
};
/*======================================================================*/
B_MESSAGE_MAP(MyWindow)
MSG_ENTRY_WL    (MyWindow,  WMMY_SETDATA_SHORT, Msg_SETDATAshort)
MSG_ENTRY_WL    (MyWindow,  WMMY_SETDATA_FLOAT, Msg_SETDATAfloat)

MSG_ENTRY       (MyWindow,  WM_CREATE,          Msg_CREATE)
MSG_ENTRY_L     (MyWindow,  WM_LBUTTONDOWN,     Msg_LBUTTONDOWN)
MSG_ENTRY_L     (MyWindow,  WM_LBUTTONDBLCLK,   Msg_LBUTTONDBLCLK)
MSG_ENTRY_L     (MyWindow,  WM_RBUTTONDOWN,     Msg_RBUTTONDOWN)
MSG_ENTRY_WL    (MyWindow,  WM_MOUSEMOVE,       Msg_MOUSEMOVE)
MSG_ENTRY       (MyWindow,  WM_SIZE,            Msg_SIZE)
E_MESSAGE_MAP()
/*======================================================================*/
bool    MyWindow::Alloc(int size)
{
    if(m_pData != 0 && m_nSize == size)
        return true;

    Free();
    if(size <= 0)
        return false;

    m_pData = new float[size];
    if(m_pData == 0)
        return false;
    m_nSize = size;
    return true;
}
/*----------------------------------------------------------------------*/
void    MyWindow::Free()
{
    if(m_pData) 
        delete m_pData;
    m_nSize = 0;
}
/*----------------------------------------------------------------------*/
MyWindow::MyWindow(IOSCE* pOsc) : MWindow() 
{
    m_pOsc = pOsc;

    m_pData = 0;
    m_nSize = 0;
    m_nCh   = 0;

    m_fYmi = -10.0f;
    m_fYma =  10.0f;
}
/*----------------------------------------------------------------------*/
MyWindow::~MyWindow()
{
    if(m_pOsc) 
        m_pOsc->Release();
    Free();
}
/*======================================================================*/
void    MyWindow::OnPaint(HDC dc, RECT& rc)
{
    if(m_pOsc == 0)
        return;

    m_pOsc->Property(OSC_PROPSET_BOTTOMSCALE_TEXT, "Sm"); 
    m_pOsc->Property(OSC_PROPSET_LEFTSCALE_TEXT,   "Bin"); 

    OSCScale pc; 
    OSCInfoE oi;
    if(m_pData != 0 && m_nSize > 0)
        pc.Set(0.0f, float(m_nSize), m_fYmi, m_fYma, m_fYmi, m_fYma);
    else
        pc.Set(0.0f, 10.0f, -10.0f, 10.0f, -10.0f, 10.0f);
    m_pOsc->BeginPaint(dc, rc, pc, 100, &oi);
    if(m_pData != 0 && m_nSize > 0)
        m_pOsc->DrawDataF(m_pData, m_nSize, m_nCh, 0, 0, 0);
    m_pOsc->EndPaint();
}
/*======================================================================*/
int     MyWindow::Msg_CREATE()
{
    OSCParamE op;
    OSCMarker om;
    op.Set(OSC_DIGITAL|OSC_FLOAT_SCALE|OSC_DIGITAL_ZOOM|OSC_FLOAT_SCALE_ZOOM,
           6, 6, 5,
           RGB(192,192,192), RGB(128,128,128),
           RGB(192,192,192), RGB(0,0,0), RGB(0,0,100), 0,
           "Courier New Cyr", 7, 2, hWnd);
    om.Set(OSC_MARKER_CLINE, RGB(128,192,128), 0,0,0);

    m_pOsc->SetParameters(&op);
    m_pOsc->Property(OSC_PROPSET_CURSOR_MARKER, &om);

    return 0;
}
/*----------------------------------------------------------------------*/
int     MyWindow::Msg_MOUSEMOVE(WPARAM w, LPARAM l)
{
    RECT rc; GetClientRect(hWnd, &rc);
    if(m_pOsc) 
        if(m_pOsc->On_MOUSEMOVE(rc, w, l)) 
            IRect(); 
    return 0;
}
/*----------------------------------------------------------------------*/
int     MyWindow::Msg_SIZE()
{       
    if(m_pOsc) 
        IRect(); 
    return 0;
}
/*----------------------------------------------------------------------*/
int     MyWindow::Msg_LBUTTONDBLCLK(LPARAM l)
{
    if(m_pOsc) 
    {
        RECT rc; GetClientRect(hWnd, &rc);
        m_pOsc->On_BUTTONDOWN(rc, MK_LBUTTON, l, 3);
    }
    IRect();
    return 0;
}
/*----------------------------------------------------------------------*/
int     MyWindow::Msg_LBUTTONDOWN(LPARAM l)
{
    if(m_pOsc) 
    {
        RECT rc; GetClientRect(hWnd, &rc);
        m_pOsc->On_BUTTONDOWN(rc, MK_LBUTTON, l, 0);
    }
    IRect();
    return 0;
}
/*----------------------------------------------------------------------*/
int     MyWindow::Msg_RBUTTONDOWN(LPARAM l)
{
    if(m_pOsc)
    {
        RECT rc; GetClientRect(hWnd, &rc);
        m_pOsc->On_BUTTONDOWN(rc, MK_RBUTTON, l);
    }
    IRect();
    return 0;
}
/*----------------------------------------------------------------------*/
void    MyWindow::OnDestroy()
{
    delete this;
}
/*======================================================================*/
int     MyWindow::Msg_SETDATAshort(WPARAM w, LPARAM l)   // LOw - size, HIw - channels, l - ptr
{
    int size  = LOWORD(w);
    int ch    = HIWORD(w);
    short* pb = (short*)l;

    if(pb == 0 || size <= 0 || ch <= 0)
        return 0;
try
{
    int blen = size * ch;
    if(Alloc(blen))
    {
        m_nSize = size;
        m_nCh   = ch;

        int mi = pb[0];
        int ma = pb[0];
        m_pData[0] = float(mi);
        for(int i=1; i<blen; ++i)
        {
            short v = pb[i];
            if(v < mi) mi = v;
            if(v > ma) ma = v;
            m_pData[i] = float(v);
        }
        float fRange10 = float(ma - mi) * 0.1f;
        if(mi < m_fYmi || mi > m_fYmi + fRange10)
            m_fYmi = float(mi) - fRange10;
        if(ma > m_fYma || ma < m_fYma - fRange10)
            m_fYma = float(ma) + fRange10;
        IRect();
    }
    return 0;
}
catch(...)
{
    return 0;
}
}
/*----------------------------------------------------------------------*/
int     MyWindow::Msg_SETDATAfloat(WPARAM w, LPARAM l)   // LOw - size, HIw - channels, l - ptr
{
    int size  = LOWORD(w);
    int ch    = HIWORD(w);
    float* pb = (float*)l;

    if(pb == 0 || size <= 0 || ch <= 0)
        return 0;
try
{
    int blen = size * ch;
    if(Alloc(blen))
    {
        m_nSize = size;
        m_nCh   = ch;

        float mi = pb[0];
        float ma = pb[0];
        m_pData[0] = mi;
        for(int i=1; i<blen; ++i)
        {
            float v = pb[i];
            if(v < mi) mi = v;
            if(v > ma) ma = v;
            m_pData[i] = v;
        }
        float fRange10 = (ma - mi) * 0.1f;
        if(mi < m_fYmi || mi > m_fYmi + fRange10)
            m_fYmi = mi - fRange10;
        if(ma > m_fYma || ma < m_fYma - fRange10)
            m_fYma = ma + fRange10;
        IRect();
    }
    return 0;
}
catch(...)
{
    return 0;
}
}
/*######################################################################*/
MWindow*    NewWindow(LPSTR str)
{
    if(!stricmp(str, "DRIVERVIEW"))
    {
        IOSCE *pOsc = MyLoader.GetOSC(); 
        if(pOsc != 0)
            return new MyWindow(pOsc);
    }
    return MyLoader.CallNextNewWindow(str);
}
/*######################################################################*/
