#pragma once

/*######################################################################*/
class   MEvent
{
private:

    HANDLE          m_hEvent;

public:

    MEvent(BOOL bManualReset = FALSE, BOOL bInitialState = FALSE, 
           LPCTSTR pName = NULL, LPSECURITY_ATTRIBUTES pSecurity = NULL)
           : m_hEvent(NULL)
    {
        m_hEvent = ::CreateEvent(pSecurity, bManualReset, bInitialState, pName);
    }

    virtual ~MEvent()
    {
        if(m_hEvent) 
            ::CloseHandle(m_hEvent);
    }

    bool            OK() const      { return m_hEvent != NULL; }
    void            Signal()        { ::SetEvent(m_hEvent);    }
    void            Reset()         { ::ResetEvent(m_hEvent);  }
    const HANDLE    Handle() const  { return m_hEvent;         }
    DWORD           WaitSignal(unsigned uTimeOut = INFINITE) const     
    { 
        return m_hEvent ? ::WaitForSingleObject(m_hEvent, uTimeOut) : 0; 
    }
    bool    WaitObject0(unsigned uTimeOut = INFINITE) const     
    { 
        return m_hEvent ? (::WaitForSingleObject(m_hEvent, uTimeOut) == WAIT_OBJECT_0) : false; 
    }
};
/*======================================================================*/
class   MOverlapped
{
private:

    OVERLAPPED      m_Ov;

public:

    MOverlapped()  
    { 
        ZeroMemory(&m_Ov, sizeof(m_Ov)); 
        m_Ov.hEvent = ::CreateEvent(0, TRUE, FALSE, 0);
    }

    MOverlapped(int init)  
    { 
        ZeroMemory(&m_Ov, sizeof(m_Ov)); 
        if(init >= 0)
            m_Ov.hEvent = ::CreateEvent(0, init == 0 ? FALSE : TRUE, FALSE, 0);
    }

    virtual ~MOverlapped() 
    { 
        if(m_Ov.hEvent)
            ::CloseHandle(m_Ov.hEvent);
    }

    DWORD   WaitSignal(unsigned uTimeOut = INFINITE) const     
    { 
        return m_Ov.hEvent ? ::WaitForSingleObject(m_Ov.hEvent, uTimeOut) : 0; 
    }
    bool    WaitObject0(unsigned uTimeOut = INFINITE) const     
    { 
        return m_Ov.hEvent ? (::WaitForSingleObject(m_Ov.hEvent, uTimeOut) == WAIT_OBJECT_0) : false; 
    }
    bool    Init(BOOL res = TRUE)
    {
        if(m_Ov.hEvent == 0)
            m_Ov.hEvent = ::CreateEvent(0, res, FALSE, 0);
        else
            Reset();
        return m_Ov.hEvent != 0;
    }
    void    Clear()
    {
        if(m_Ov.hEvent)
            ::CloseHandle(m_Ov.hEvent);
        ZeroMemory(&m_Ov, sizeof(m_Ov)); 
    }
    void    Reset()         
    { 
        if(m_Ov.hEvent)
            ::ResetEvent(m_Ov.hEvent);
    }
    const HANDLE    Handle() const  { return m_Ov.hEvent; }
    OVERLAPPED&     Get()           { return m_Ov;        }
};
/*======================================================================*/
class   MCriticalSection
{
private:

    CRITICAL_SECTION    m_CritSec; 

public:

    MCriticalSection()
    { 
        InitializeCriticalSection(&m_CritSec); 
    }

    virtual ~MCriticalSection()
    { 
        DeleteCriticalSection(&m_CritSec); 
    }

    void    Enter()
    {
        try{ EnterCriticalSection(&m_CritSec); }
        catch(...){ ; }
    };

    void    Leave()
    {
        try{ LeaveCriticalSection(&m_CritSec); }
        catch(...){ ; }
    };
};
/*----------------------------------------------------------------------*/
class   MLocker
{
private:

    MCriticalSection&  m_rCS;

public:

    MLocker(MCriticalSection& cs) : m_rCS(cs)
    { 
        m_rCS.Enter(); 
    }

    virtual ~MLocker()
    { 
        m_rCS.Leave(); 
    }
};
/*######################################################################*/
