#ifndef __MFILEMAP_H__
#define __MFILEMAP_H__
/*######################################################################*/
class   MFileMap
{
private:
    HANDLE      m_hFile;
    HANDLE      m_hMap;
    void*       m_Ptr;
    unsigned    m_nSize;
    long        m_nErr;

public:
    MFileMap()  
    { 
        m_Ptr = 0; 
        m_nSize = 0; 
        m_hFile = m_hMap = INVALID_HANDLE_VALUE; 
    }
    ~MFileMap() 
    { 
        Close(); 
    }
/*----------------------------------------------------------------------*/
void        Close()
{
    if(m_Ptr != 0)   
        ::UnmapViewOfFile(m_Ptr);
    if(m_hMap  != INVALID_HANDLE_VALUE)  
        ::CloseHandle(m_hMap);
    if(m_hFile != INVALID_HANDLE_VALUE) 
        ::CloseHandle(m_hFile);
    m_Ptr   = 0;
    m_hFile = m_hMap = INVALID_HANDLE_VALUE; 
    m_nSize = 0;
}
/*----------------------------------------------------------------------*/
bool        Create(TCHAR const* path, unsigned size, TCHAR const* mapName = 0)
{
    Close(); 
    if(size == 0) 
        return false;
    if(path && *path)
    {
        m_hFile = ::CreateFile(path, GENERIC_WRITE|GENERIC_READ, 
                               FILE_SHARE_READ|FILE_SHARE_WRITE, 0,
                               CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
        if(m_hFile == INVALID_HANDLE_VALUE)
            return false;

        m_hMap = ::CreateFileMapping(m_hFile, 0, PAGE_READWRITE, 0, size, mapName);
    }
    else
        m_hMap = ::CreateFileMapping((HANDLE)-1, 0, PAGE_READWRITE, 0, size, mapName);
    
    if(m_hMap == 0) 
    { 
        m_nErr = ::GetLastError(); 
        return false; 
    }
    m_Ptr = ::MapViewOfFile(m_hMap, FILE_MAP_READ|FILE_MAP_WRITE, 0, 0, 0);
    if(m_Ptr == 0)  
    { 
        m_nErr = ::GetLastError(); 
        return false; 
    }
    m_nSize = size;
    return true;
}
/*----------------------------------------------------------------------*/
bool        Open(TCHAR const* path, bool bShare = false, TCHAR const* mapName = 0)
{
    Close(); 
    if(path && *path)
    {
        m_hFile = ::CreateFile(path, GENERIC_WRITE|GENERIC_READ, 
                               bShare ? FILE_SHARE_READ|FILE_SHARE_WRITE : 0,
                               0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
        if(m_hFile == INVALID_HANDLE_VALUE) 
            return false;
        m_nSize = ::GetFileSize(m_hFile, 0);
        m_hMap = ::CreateFileMapping(m_hFile, 0,  FILE_MAP_READ|FILE_MAP_WRITE, 0, 0, 0);
    }
    else
    {
        m_hMap = ::OpenFileMapping(FILE_MAP_READ|FILE_MAP_WRITE, 0, mapName);
        m_nSize = ::GetFileSize(m_hMap, 0);
    }
    if(m_hMap == INVALID_HANDLE_VALUE) 
    { 
        m_nErr = ::GetLastError(); 
        return false; 
    }
    m_Ptr = ::MapViewOfFile(m_hMap, FILE_MAP_READ|FILE_MAP_WRITE, 0, 0, 0);
    if(m_Ptr == 0)  
    { 
        m_nErr = ::GetLastError(); 
        return false; 
    }
    return true;
}
/*----------------------------------------------------------------------*/
bool        OK()        { return m_Ptr != 0; }
void*       GetPtr()    { return m_Ptr;      }
unsigned    GetSize()   { return m_nSize;    }
long        GetError()  { return m_nErr;     }
};
/*######################################################################*/
#endif // __MFILEMAP_H__
