#pragma once

#include    <crtdbg.h>
/*######################################################################*/
extern "C"
{
    LONG  __cdecl _InterlockedIncrement(LONG volatile* v);
    LONG  __cdecl _InterlockedDecrement(LONG volatile* v);
    LONG  __cdecl _InterlockedCompareExchange(LONG volatile* v, LONG e, LONG c);
    LONG  __cdecl _InterlockedExchange(LONG volatile* v, LONG e);
    LONG  __cdecl _InterlockedExchangeAdd(LONG volatile* v, LONG e);
}

#pragma intrinsic (_InterlockedCompareExchange)
#define InterlockedCompareExchange _InterlockedCompareExchange

#pragma intrinsic (_InterlockedExchange)
#define InterlockedExchange _InterlockedExchange 

#pragma intrinsic (_InterlockedExchangeAdd)
#define InterlockedExchangeAdd _InterlockedExchangeAdd

#pragma intrinsic (_InterlockedIncrement)
#define InterlockedIncrement _InterlockedIncrement

#pragma intrinsic (_InterlockedDecrement)
#define InterlockedDecrement _InterlockedDecrement
/*######################################################################*/
template <class T> class MInterlocked
{
public:
    
    static T    Increment(T& v) throw()
    {
        _ASSERT( sizeof(v) == 4 );
        LONG r = InterlockedIncrement(reinterpret_cast<long*>(&v));
        return static_cast<T>(r);
    }

    static T    Decrement(T& v) throw()
    {
        _ASSERT( sizeof(v) == 4 );
        LONG r = InterlockedDecrement(reinterpret_cast<long*>(&v));
        return static_cast<T>(r);
    }

    static T    Exchange(T& v, T e) throw()
    {
        _ASSERT( sizeof(v) == 4 );
        LONG r = InterlockedExchange(reinterpret_cast<long*>(&v), 
                                     static_cast<long>(e));
        return static_cast<T>(r);
    }

    static T    CompareExchange(T& v, T e, T c) throw()
    {
        _ASSERT( sizeof(v) == 4 );
        LONG r = InterlockedCompareExchange(reinterpret_cast<long*>(&v), 
                                            static_cast<long>(e),
                                            static_cast<long>(c));
        return static_cast<T>(r);
    }

    static T    ExchangeAdd(T& v, T e) throw()
    {
        _ASSERT( sizeof(v) == 4 );
        LONG r = InterlockedExchangeAdd(reinterpret_cast<long*>&(v), 
                                        static_cast<long>(e));
        return static_cast<T>(r);
    }
};
/*======================================================================*/
class   MLock
{
private:

    volatile long   m_state;

public:

    MLock() : m_state(0) {}

    bool        Lock()
    {
        long val = InterlockedExchange(&m_state, 1);
        return (val == 1) ? false : true;
    }

    void        Unlock() 
    { 
        InterlockedExchange(&m_state, 0); 
    }
};
/*######################################################################*/
