#pragma once

#include    "MInterlocked.h"
#include    "MEvent.h"
/*######################################################################*/
class   MThread
{
private:

    DWORD           m_ident;
    volatile HANDLE m_hThread;
    volatile int    m_bRun;

public:

    volatile int    m_nReturn;

    MThread()  
    { 
        m_hThread = NULL; 
        m_ident   = 0; 
        m_bRun    = 0; 
        m_nReturn = 0;
    }
    virtual ~MThread() 
    { 
        Stop(); 
    }

    void    Init(LPTHREAD_START_ROUTINE tp, LPVOID p,
                 int mode = THREAD_PRIORITY_NORMAL)
    {
        m_hThread = CreateThread(NULL, 0, tp, p, CREATE_SUSPENDED, &m_ident);
        if(m_hThread) 
            SetThreadPriority(m_hThread, mode);
        m_nReturn = 0;
    }

    void    Stop(unsigned tout = INFINITE)
    {
        if(MInterlocked<int>::CompareExchange((int)m_bRun, 0, 1))
        {
            if(m_hThread)
            {
                ResumeThread(m_hThread);
                if(WaitForSingleObject(m_hThread, tout) != WAIT_OBJECT_0) 
                    TerminateThread(m_hThread, -1);
                m_nReturn = 0;
            }
        }
        Clear();
    }

    void    Run()         
    { 
        if(m_hThread) 
        { 
            MInterlocked<int>::Exchange((int)m_bRun, 1);
            ResumeThread(m_hThread); 
        }
    }

    int     Wait(int tout)
    {
        if(m_hThread == 0)
            return -1;
        if(WaitForSingleObject(m_hThread, tout) == WAIT_OBJECT_0) 
            return 1;
        return 0;
    }

    bool            OK() const      { return m_hThread != NULL; }
    bool            Get() const     { return m_bRun != 0;       }
    const HANDLE    Handle() const  { return m_hThread;         }
    void            Clear()         { m_hThread = NULL;         }
};
/*######################################################################*/
