#ifndef __REGISTRY_H__
#define __REGISTRY_H__
/*######################################################################*/
#include <windows.h>
#include <Tchar.h>
#include <string.h>
/*######################################################################*/
class RegKey
{
private:
    HKEY    m_hKey;
    void    Free()  { if(m_hKey) RegCloseKey(m_hKey); m_hKey = 0; }

public:
    RegKey()        { m_hKey = 0; }
    ~RegKey()       { Free();     }

    HKEY GetKey()   { return m_hKey; }
    bool IsValid()  { return m_hKey != 0; }
/*======================================================================*/
bool    Create(HKEY hParent, TCHAR const* path, REGSAM rsam = KEY_ALL_ACCESS)
{
    if(hParent == 0 || path == 0) return false;
    Free(); 
    DWORD disposition;
    if(RegCreateKeyEx(hParent, path, 0, 0,
                      REG_OPTION_NON_VOLATILE, rsam, 0, &m_hKey,
                      &disposition) == ERROR_SUCCESS) return true;
    m_hKey = 0; 
    return false;
}
/*----------------------------------------------------------------------*/
bool    Open(HKEY hParent, TCHAR const* path, REGSAM rsam = KEY_ALL_ACCESS)
{
    if(hParent == 0 || path == 0) return false;
    Free();
    if(RegOpenKeyEx(hParent, path, 0, rsam,
                    &m_hKey) == ERROR_SUCCESS) return true;
    m_hKey = 0;
    return false;
}
/*----------------------------------------------------------------------*/
bool    GetData(TCHAR const* name, void* data, int size, int* retBytes)
{
    if(m_hKey == 0 || name == 0 || data == 0 || size <= 0) return false;
    DWORD dwType, dwSize = size;
    LONG r = RegQueryValueEx(m_hKey, name, 0, &dwType,
                             (unsigned char*)data, &dwSize);
    if(r == ERROR_SUCCESS && dwType > 0 && dwType <= REG_RESOURCE_REQUIREMENTS_LIST)
    {
        if(retBytes) *retBytes = dwSize;
        return true;
    }
    return false;
}
/*----------------------------------------------------------------------*/
bool    SetData(TCHAR const* name, void* data, int size)
{
    if(m_hKey == 0 || name == 0 || data == 0 || size <= 0) return false;
    LONG r = RegSetValueEx(m_hKey, name, 0, REG_BINARY,
                           (unsigned char*)data, size);
    return r == ERROR_SUCCESS;
}
/*----------------------------------------------------------------------*/
bool    GetString(TCHAR const* name, TCHAR* data, int size)
{
    if(m_hKey == 0 || name == 0 || data == 0) return false;
    DWORD dwType, dwSize = (size-1) * sizeof(TCHAR);
    LONG r = RegQueryValueEx(m_hKey, name, 0, &dwType,
                             (unsigned char*)data, &dwSize);
    data[dwSize] = 0;
    return r == ERROR_SUCCESS && dwType == REG_SZ;
}
/*----------------------------------------------------------------------*/
bool    SetString(TCHAR const* name, TCHAR const* data)
{
    if(m_hKey == 0 || name == 0 || data == 0) return false;
    LONG r = RegSetValueEx(m_hKey, name, 0, REG_SZ,
                           (unsigned char*)data, (strlen(data)+1) * sizeof(TCHAR));
    return r == ERROR_SUCCESS;
}
/*----------------------------------------------------------------------*/
bool    GetChildName(int index, TCHAR* data, int size)
{
    if(data == 0) return false;
    DWORD dwSize = (size-1) * sizeof(TCHAR);
    FILETIME filetime;
    LONG r = RegEnumKeyEx(m_hKey, index, data, &dwSize,
                          0, 0, 0, &filetime);
    data[dwSize] = 0;
    return r == ERROR_SUCCESS;
}
};
/*######################################################################*/
#endif
