#include    <Windows.h>
/*######################################################################*/
#ifndef DIM
#define DIM(a)      (sizeof(a) / sizeof(a[0]))
#endif
/*######################################################################*/
#pragma pack(1)
/*----------------------------------------------------------------------*/
struct  ALFHEADER
{
private:
    char            m_sRecordName[56];

    void    Fill(char* name)
    {
        for(int i = 0; i < sizeof(m_sRecordName); ++i)
            m_sRecordName[i] = (name && *name) ? *name++ : ' ';
        m_nRecordSize = 0L;
    }

public:
    __int64         m_nRecordSize;

    ALFHEADER(char* name)
    {
        Fill(name);
    }

    ALFHEADER()
    {
        Fill("ADCLABFF");
    }
};
/*----------------------------------------------------------------------*/
struct  SAMPLESFMT : ALFHEADER
{
    enum MaskFlags
    {
        SFMT_CHANNELS_COUNT             = 0x01,
        SFMT_FREQUENCY                  = 0x02,
        SFMT_TYPE                       = 0x04,
    };

    enum SamplesType
    {
        SAMPLES_TYPE_FLOAT              = 1,
        SAMPLES_TYPE_DOUBLE             = 2,
        SAMPLES_TYPE_SIGNED_INT8        = 3,
        SAMPLES_TYPE_UNSIGNED_INT8      = 4,
        SAMPLES_TYPE_SIGNED_INT16       = 5,
        SAMPLES_TYPE_UNSIGNED_INT16     = 6,
        SAMPLES_TYPE_SIGNED_INT32       = 7,
        SAMPLES_TYPE_UNSIGNED_INT32     = 8,
    };

    unsigned        m_mask;
    union
    {
    int             m_channelsCount;
    float           m_fCommonRange;
    };
    double          m_frequencyPerChannel;
    char            m_samplesType;
    char            m_lowerSignificantBit;
    char            m_significantBitsCount;
    char            m_reserved0;

    void    Fill(float fRange, double freq, char format)
    {
        if(fRange > 0.0f)
            m_fCommonRange  = -fRange;
        else
            m_channelsCount = 0;
        m_frequencyPerChannel = freq;
        m_samplesType = format ? SAMPLES_TYPE_SIGNED_INT16 : SAMPLES_TYPE_UNSIGNED_INT16;
    }

    SAMPLESFMT() : ALFHEADER("SAMPLES_FORMAT") 
    {
        m_nRecordSize = sizeof(SAMPLESFMT) - sizeof(ALFHEADER);

        m_mask                  = SFMT_CHANNELS_COUNT | SFMT_FREQUENCY | SFMT_TYPE;
        m_channelsCount         = 0;
        m_frequencyPerChannel   = 0.0;
        m_samplesType           = SAMPLES_TYPE_UNSIGNED_INT16;
        m_lowerSignificantBit   = 0;
        m_significantBitsCount  = 16;
        m_reserved0             = 0;
    }
};
/*----------------------------------------------------------------------*/
struct  SAMPLES_RECORD : ALFHEADER
{
    SAMPLES_RECORD() : ALFHEADER("SAMPLES_RECORD")
    {
        m_nRecordSize = -1L;
    }
};
/*======================================================================*/
struct  SD_FORMAT : ALFHEADER
{
    unsigned        m_nFreq;
    char            m_nMode;
    char            m_nChannels;
    unsigned short  m_nChannel[32];

    void    Fill(unsigned freq, int nch)
    {
        m_nFreq     = 0;
        m_nMode     = 0;
        m_nChannels = 0;

        if(freq && nch > 0)
        {
            m_nFreq     = freq;
            m_nMode     = 0;
            m_nChannels = nch > DIM(m_nChannel) ? DIM(m_nChannel) : nch;
        }
    }

    void    FillChannel(unsigned index, 
                        unsigned short ch, unsigned short gain, unsigned short mode)
    {
        if(index < DIM(m_nChannel))
            m_nChannel[index] = (ch & (32-1)) | (gain << 8) | (mode << 8+6);
    }

    SD_FORMAT() : ALFHEADER("SD_FORMAT")
    {
        m_nRecordSize = sizeof(SD_FORMAT) - sizeof(ALFHEADER);

        Fill(0, 0);      
        for(int i = 0; i < DIM(m_nChannel); ++i)
            m_nChannel[i] = 0;
    }
};
/*----------------------------------------------------------------------*/
struct  SD_INFO : ALFHEADER
{
    char            m_nFormat;
    char            m_nSize;
    char            m_nCalData[100];

    SD_INFO() : ALFHEADER("SD_INFO")
    {
        m_nRecordSize = sizeof(SD_INFO) - sizeof(ALFHEADER);
        
        m_nFormat = 0;
        m_nSize   = 0;
        for(int i = 0; i < DIM(m_nCalData); ++i)
            m_nCalData[i] = char(0xFF);
    }
};
/*----------------------------------------------------------------------*/
struct  ALFFileHeader
{
//         :
//  ALFHEADER, SD_FORMAT, SD_INFO, SAMPLESFMT, SAMPLES_RECORD
//     ,    ,
//       .  
//    EL100SD.
//
//     SD_FORMAT::FillChannel(index, channel, gain, mode)
//  index   0 .. 31
//  channel 0 .. 31
//  gain(0) = 0, gain(1) = 1, gain(2) = 10, gain(3) = 100, gain(4) = 1000 
//  mode(0) = ZERO, mode(1) = SINGLE, mode(2) = DIFF

    ALFHEADER       alfh; 
    SD_FORMAT       sdfm;
    SD_INFO         sdif;
    SAMPLESFMT      sfmt; 
    SAMPLES_RECORD  srec;

    void    Save(HANDLE hFile)
    {
        if(hFile == INVALID_HANDLE_VALUE)
            return;
        static DWORD dw = 0;
        ::WriteFile(hFile, this, sizeof(*this), &dw, 0);
    }
};
/*----------------------------------------------------------------------*/
#pragma pack()
/*######################################################################*/
HANDLE  CreateALFFile(char* fpath, int number)
{
    char name[256];
    wsprintf(name, "%s%04dData.ALF", fpath ? fpath : "", number);
    return ::CreateFile(name, GENERIC_WRITE, 0, 0,
                        CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, 0);
}
/*----------------------------------------------------------------------*/
int     SaveALFHeader(HANDLE hFile,
                      double fRange,
                      unsigned nChan, int* pChannel, int* pGains,
                      double freq, char format)
{
    if(hFile == INVALID_HANDLE_VALUE)
        return -1;
    if(pChannel == 0 || pGains == 0 || nChan == 0)
        return 0;

    ALFFileHeader alf;

    alf.sdfm.Fill(unsigned(freq + 0.5), nChan);
    for(unsigned i = 0; i < nChan; ++i)
    {
        unsigned short gain = pGains[i] & 0xFFFF;
        unsigned short mode = pGains[i] & ADC_CHANNEL_MODEDIFF ? 2 : 1;
        switch(gain)
        {
        default:
        case 0:     gain = 0, mode = 0; // Zero
            break;
        case 1:     gain = 0;
            break;
        case 10:    gain = 1;
            break;
        case 100:   gain = 2;
            break;
        case 200:   gain = 3;
            break;
        }
        alf.sdfm.FillChannel(i, pChannel[i], gain, mode);
    }
                                   
    alf.sfmt.Fill(float(fRange), freq, format);

    alf.Save(hFile);
    return 1;
}
/*----------------------------------------------------------------------*/
int     SaveALFFile(char* fpath, int number, 
                    double fRange,
                    unsigned channels, int* pChannel, int* pGains,
                    unsigned sizeKCh, double freq)
{
    HANDLE hFile = CreateALFFile(fpath, number);
    if(hFile == INVALID_HANDLE_VALUE) 
        return -1;

    if(SaveALFHeader(hFile, fRange, channels, pChannel, pGains, freq, 0) <= 0)
        return 0;

    unsigned nbuff = channels * 1024;
    unsigned short* buff = new unsigned short[nbuff];
    for(unsigned i = 0; i < nbuff; ++i)
        buff[i] = 0x0000;

    for(i = 0; i < sizeKCh; ++i)
    {
        static DWORD dw = 0;
        ::WriteFile(hFile, buff, nbuff * sizeof(buff[0]), &dw, 0);
    }
    delete buff;

    ::CloseHandle(hFile); 
    return 1;
}
/*######################################################################*/
