#ifndef __ADC_DEVICE_H__
#define __ADC_DEVICE_H__

#include "ADC_CONST.h"

//////////////////////////////////////////////////////////////////////////
// Revision : 28-08-2006
//////////////////////////////////////////////////////////////////////////

#pragma pack(1)
/*######################################################################*/
/* LOWORD is major version, HIWORD is minor version                     */
/*                          minor|major                                 */ 
#define MAJOR_VERSION       0x000F  // essential change
#define MINOR_VERSION       0x0001  // not essential change
#define CURRENT_VERSION     (MAJOR_VERSION | (MINOR_VERSION << 16))
/*######################################################################*/
struct  ADCParametersBase
{
    int         m_nType;
    int         m_nSizeOf;
};
/*----------------------------------------------------------------------*/
struct  IADCDevice
{
    virtual unsigned __stdcall  AddRef() = 0;
    virtual unsigned __stdcall  Release() = 0;

    virtual int      __stdcall  Setup(int baseAdr, int DRQ, int IRQ, HANDLE hEvent) = 0;
    virtual int      __stdcall  Test() = 0;
    virtual int      __stdcall  Init(int mode, ADCParametersBase*, float*) = 0;
    virtual int      __stdcall  Start() = 0;
    virtual void     __stdcall  Stop() = 0;
    virtual int      __stdcall  GetData(int mode, void* pAdr, int size, int offset) = 0;
    virtual int      __stdcall  PortIO(int mode, void* pAdr, int size) = 0;
    virtual int      __stdcall  Get(int mode, void* value) = 0;
    virtual void     __stdcall  ClearInterrupt() = 0;
};
/*######################################################################*/
// m_nType = ADC_DMA_TYPE
struct  ADCParametersDMA : public ADCParametersBase
{
    int         m_nStartOf;
    int         m_nIntOf;

    int         m_nDMABlockSize;
    int         m_nDMAMode;

    float       m_fFreqStart;
    int         m_nTimerStart;
    float       m_fFreqPack;
    int         m_nTimerPack;
    int         m_nFirstChannel;
    int         m_nChannelNumber;
    int         m_nGain;

    ADCParametersDMA()
    {
        m_nType   = ADC_DMA_TYPE;
        m_nSizeOf = sizeof(ADCParametersDMA);
    }
};
/*######################################################################*/
// m_nType = ADC_DMAEX_TYPE
struct  ADCParametersDMAEX : public ADCParametersBase
{
    int*        m_pnGains;
    int*        m_pnChannels;
    int         m_nSize;
    int         m_nControl;
    int         m_nSyncLevel[2];

    ADCParametersDMAEX()
    {
        m_nType   = ADC_DMAEX_TYPE;
        m_nSizeOf = sizeof(ADCParametersDMAEX);
    }
};
/*######################################################################*/
// m_nType = ADC_CALIBRATION_TYPE
struct  ADCParametersCALIBRATION : public ADCParametersBase
{
    unsigned    m_nFlags;
    int         m_nADC;             // sise of ADC table
    int*        m_pnADCGain;
    double*     m_pADCOffset;
    double*     m_pADCGain;
    int         m_nDAC;             // sise of DAC table
    double*     m_pDACOffset;
    double*     m_pDACGain;
    unsigned    m_nExtra;

    ADCParametersCALIBRATION()
    {
        m_nType   = ADC_CALIBRATION_TYPE;
        m_nSizeOf = sizeof(ADCParametersCALIBRATION);
        m_nFlags  = 0;
    }
};
/*######################################################################*/
// m_nType = ADC_GENERATOR_TYPE
struct  ADCParametersGENERATOR : public ADCParametersBase
{
    double      m_fFreq;            // 0 - 50 KHz
    int         m_nBank;            // 0 - 32 (256 words)
    union
    {
    int         m_nAmplitude;       // 0 - 5
    int         m_nSize1;           // PCA1 Length
    };
    union
    {
    int         m_nSize;            // size of DAC table
    int         m_nSize2;           // PCA2 Length
    };
    short*      m_nData;            //

    ADCParametersGENERATOR()
    {
        m_nType   = ADC_GENERATOR_TYPE;
        m_nSizeOf = sizeof(ADCParametersGENERATOR);
    }
};
/*######################################################################*/
// m_nType = ADC_SAMPLE_TYPE
struct  ADCParametersSAMPLE : public ADCParametersBase
{
    int         m_nControl;
    int         m_nGain;
    int         m_nChannel;
    int         m_nValue;

    ADCParametersSAMPLE()
    {
        m_nType   = ADC_SAMPLE_TYPE;
        m_nSizeOf = sizeof(ADCParametersSAMPLE);
    }
};
/*######################################################################*/
// m_nType = ADC_BLOCK_TYPE
struct  ADCParametersBLOCK : public ADCParametersBase
{
    int         m_nStartOf;

    int         m_nBlockSize;
    int         m_nMode;
    int         m_nControl;
    int         m_nLevel;
    double      m_fFreqStart;
    double      m_fFreqPack;
    int         m_nChannelSize;
    int*        m_pnGains;
    int*        m_pnChannels;

    ADCParametersBLOCK()
    {
        m_nType   = ADC_BLOCK_TYPE;
        m_nSizeOf = sizeof(ADCParametersBLOCK);
    }
};
/*######################################################################*/
// m_nType = ADC_STREAM_TYPE
struct  ADCParametersSTREAM : public ADCParametersBase
{
    int         m_nStartOf;

    int         m_nBlockSize;
    int         m_nDataBuffers;
    short**     m_pDataBuffers;

    int         m_nMode;
    int         m_nControl;
    int         m_nLevel;
    double      m_fFreqStart;
    double      m_fFreqPack;
    int         m_nChannelSize;
    int*        m_pnGains;
    int*        m_pnChannels;

    ADCParametersSTREAM()
    {
        m_nType   = ADC_STREAM_TYPE;
        m_nSizeOf = sizeof(ADCParametersSTREAM);
    }
};
/*######################################################################*/
#pragma pack()
#endif
